==> usbrumix2-ruby1.8.7-2.00.zip: ruby/1.8.7/i386-mingw32/lib/ruby/gems/1.8/gems/bitclust-core-0.8.0/lib/bitclust/functionreferenceparser.rb <==
#
# bitclust/functionreferenceparser.rb
#
# Copyright (c) 2006-2007 Minero Aoki
#
# This program is free software.
# You can distribute/modify this program under the Ruby License.
#

require 'bitclust/exception'

module BitClust

  # Parser for C API reference file (refm/capi/src/*)
  # Much simpler than Ruby API reference parser(RRDParser)
  # because C APIs does not have library, class, etc.
  class FunctionReferenceParser

    # Returns an array of FunctionEntry
    def FunctionReferenceParser.parse_file(path, params = {"version" => "1.9.0"})
      parser = new(FunctionDatabase.dummy(params))
      parser.parse_file(path, File.basename(path, ".rd"), params)
    end

    def initialize(db)
      @db = db
    end

    def parse_file(path, filename, properties)
      fopen(path, 'r:UTF-8') {|f|
        return parse(f, filename)
      }
    end

    def parse(f, filename)
      @filename = filename
      file_entries LineInput.new(f)
      @db.functions
    end

    private

    def file_entries(f)
      f.skip_blank_lines
      f.while_match(/\A---/) do |header|
        entry header.sub(/\A---/, '').strip, f.break(/\A---/)
        f.skip_blank_lines
      end
    end

    def entry(header, body)
      h = parse_header(header)
      @db.open_function(h.name) {|f|
        f.filename = @filename
        f.macro = h.macro
        f.private = h.private
        f.type = h.type
        f.name = h.name
        f.params = h.params
        f.source = body.join('')
      }
    end

    def parse_header(header)
      h = FunctionHeader.new
      m = header.match(/\A\s*(MACRO\s+)?(static\s+)?(.+?\W)(\w+)(\(.*\))?\s*\z/)
      raise ParseError, "syntax error: #{header.inspect}" unless m
      h.macro = m[1] ? true : false
      h.private = m[2] ? true : false
      h.type = m[3].strip
      h.name = m[4]
      h.params = m[5].strip if m[5]
      h
    end

  end

  # Represents C function signature
  # (Used internally in this file)
  FunctionHeader = Struct.new(:macro, :private, :type, :name, :params)

end

==> usbrumix2-ruby1.9.3-2.20.zip: ruby/1.9.3/i386-mingw32/lib/ruby/gems/1.9.1/gems/bitclust-core-0.9.0/lib/bitclust/functionreferenceparser.rb <==
#
# bitclust/functionreferenceparser.rb
#
# Copyright (c) 2006-2007 Minero Aoki
#
# This program is free software.
# You can distribute/modify this program under the Ruby License.
#

require 'bitclust/exception'

module BitClust

  # Parser for C API reference file (refm/capi/src/*)
  # Much simpler than Ruby API reference parser(RRDParser)
  # because C APIs does not have library, class, etc.
  class FunctionReferenceParser

    # Returns an array of FunctionEntry
    def FunctionReferenceParser.parse_file(path, params = {"version" => "1.9.0"})
      parser = new(FunctionDatabase.dummy(params))
      parser.parse_file(path, File.basename(path, ".rd"), params)
    end

    def initialize(db)
      @db = db
    end

    def parse_file(path, filename, properties)
      fopen(path, 'r:UTF-8') {|f|
        return parse(f, filename, properties)
      }
    end

    def parse(f, filename, params = {})
      @filename = filename
      s = Preprocessor.read(f, params)
      file_entries LineInput.for_string(s)
      @db.functions
    end

    private

    def file_entries(f)
      f.skip_blank_lines
      f.while_match(/\A---/) do |header|
        entry header.sub(/\A---/, '').strip, f.break(/\A---/)
        f.skip_blank_lines
      end
    end

    def entry(header, body)
      h = parse_header(header)
      @db.open_function(h.name) {|f|
        f.filename = @filename
        f.macro = h.macro
        f.private = h.private
        f.type = h.type
        f.name = h.name
        f.params = h.params
        f.source = body.join('')
      }
    end

    def parse_header(header)
      h = FunctionHeader.new
      m = header.match(/\A\s*(MACRO\s+)?(static\s+)?(.+?\W)(\w+)(\(.*\))?\s*\z/)
      raise ParseError, "syntax error: #{header.inspect}" unless m
      h.macro = m[1] ? true : false
      h.private = m[2] ? true : false
      h.type = m[3].strip
      h.name = m[4]
      h.params = m[5].strip if m[5]
      h
    end

  end

  # Represents C function signature
  # (Used internally in this file)
  FunctionHeader = Struct.new(:macro, :private, :type, :name, :params)

end
